package balance

import (
	"context"
	"encoding/hex"

	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/logs"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/innerring/processors"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	balanceEvent "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event/balance"
	"go.uber.org/zap"
)

func (bp *Processor) handleLock(ctx context.Context, ev event.Event) {
	lock := ev.(balanceEvent.Lock)
	bp.log.Info(ctx, logs.Notification,
		zap.String("type", "lock"),
		zap.String("value", hex.EncodeToString(lock.ID())))

	// send an event to the worker pool

	err := processors.SubmitEvent(bp.pool, bp.metrics, "lock", func() bool {
		return bp.processLock(ctx, &lock)
	})
	if err != nil {
		// there system can be moved into controlled degradation stage
		bp.log.Warn(ctx, logs.BalanceBalanceWorkerPoolDrained,
			zap.Int("capacity", bp.pool.Cap()))
	}
}
