package netmap

import (
	"context"

	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/logs"
	v2netmap "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/api/netmap"
	"github.com/nspcc-dev/neo-go/pkg/crypto/keys"
	"go.uber.org/zap"
)

func (np *Processor) processNetmapCleanupTick(ctx context.Context, ev netmapCleanupTick) bool {
	if !np.alphabetState.IsAlphabet(ctx) {
		np.log.Info(ctx, logs.NetmapNonAlphabetModeIgnoreNewNetmapCleanupTick)

		return true
	}

	err := np.netmapSnapshot.forEachRemoveCandidate(ev.epoch, func(s string) error {
		key, err := keys.NewPublicKeyFromString(s)
		if err != nil {
			np.log.Warn(ctx, logs.NetmapCantDecodePublicKeyOfNetmapNode,
				zap.String("key", s))

			return nil
		}

		np.log.Info(ctx, logs.NetmapVoteToRemoveNodeFromNetmap, zap.String("key", s))

		// In notary environments we call UpdateStateIR method instead of UpdateState.
		// It differs from UpdateState only by name, so we can do this in the same form.
		// See https://github.com/nspcc-dev/frostfs-contract/issues/225
		const methodUpdateStateNotary = "updateStateIR"

		err = np.netmapClient.MorphNotaryInvoke(
			ctx,
			np.netmapClient.ContractAddress(),
			0,
			uint32(ev.epoch),
			nil,
			methodUpdateStateNotary,
			int64(v2netmap.Offline), key.Bytes(),
		)
		if err != nil {
			np.log.Error(ctx, logs.NetmapCantInvokeNetmapUpdateState, zap.Error(err))
		}

		return nil
	})
	if err != nil {
		np.log.Warn(ctx, logs.NetmapCantIterateOnNetmapCleanerCache,
			zap.Error(err))
		return false
	}

	return true
}
