package blobovniczatree

import (
	"context"
	"testing"

	objectCore "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/core/object"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/common"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/internal/blobstortest"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/shard/mode"
	"github.com/stretchr/testify/require"
)

func TestObjectsAvailableAfterDepthAndWidthEdit(t *testing.T) {
	t.Parallel()

	rootDir := t.TempDir()

	blz := NewBlobovniczaTree(
		context.Background(),
		WithBlobovniczaShallowDepth(3),
		WithBlobovniczaShallowWidth(5),
		WithRootPath(rootDir),
	)

	require.NoError(t, blz.Open(mode.ComponentReadWrite))
	require.NoError(t, blz.Init())

	obj35 := blobstortest.NewObject(10 * 1024)
	addr35 := objectCore.AddressOf(obj35)
	raw, err := obj35.Marshal()
	require.NoError(t, err)

	pRes35, err := blz.Put(context.Background(), common.PutPrm{
		Address: addr35,
		Object:  obj35,
		RawData: raw,
	})
	require.NoError(t, err)

	gRes, err := blz.Get(context.Background(), common.GetPrm{
		Address:   addr35,
		StorageID: pRes35.StorageID,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj35, gRes.Object)

	gRes, err = blz.Get(context.Background(), common.GetPrm{
		Address: addr35,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj35, gRes.Object)

	require.NoError(t, blz.Close(context.Background()))

	// change depth and width
	blz = NewBlobovniczaTree(
		context.Background(),
		WithBlobovniczaShallowDepth(5),
		WithBlobovniczaShallowWidth(2),
		WithRootPath(rootDir),
	)

	require.NoError(t, blz.Open(mode.ComponentReadWrite))
	require.NoError(t, blz.Init())

	gRes, err = blz.Get(context.Background(), common.GetPrm{
		Address:   addr35,
		StorageID: pRes35.StorageID,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj35, gRes.Object)

	gRes, err = blz.Get(context.Background(), common.GetPrm{
		Address: addr35,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj35, gRes.Object)

	obj52 := blobstortest.NewObject(10 * 1024)
	addr52 := objectCore.AddressOf(obj52)
	raw, err = obj52.Marshal()
	require.NoError(t, err)

	pRes52, err := blz.Put(context.Background(), common.PutPrm{
		Address: addr52,
		Object:  obj52,
		RawData: raw,
	})
	require.NoError(t, err)

	require.NoError(t, blz.Close(context.Background()))

	// change depth and width back
	blz = NewBlobovniczaTree(
		context.Background(),
		WithBlobovniczaShallowDepth(3),
		WithBlobovniczaShallowWidth(5),
		WithRootPath(rootDir),
	)
	require.NoError(t, blz.Open(mode.ComponentReadWrite))
	require.NoError(t, blz.Init())

	gRes, err = blz.Get(context.Background(), common.GetPrm{
		Address:   addr35,
		StorageID: pRes35.StorageID,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj35, gRes.Object)

	gRes, err = blz.Get(context.Background(), common.GetPrm{
		Address: addr35,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj35, gRes.Object)

	gRes, err = blz.Get(context.Background(), common.GetPrm{
		Address:   addr52,
		StorageID: pRes52.StorageID,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj52, gRes.Object)

	gRes, err = blz.Get(context.Background(), common.GetPrm{
		Address: addr52,
	})
	require.NoError(t, err)
	require.EqualValues(t, obj52, gRes.Object)

	require.NoError(t, blz.Close(context.Background()))
}
