//go:build linux && integration

package fstree

import (
	"context"
	"errors"
	"os"
	"testing"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/blobstor/common"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/shard/mode"
	"github.com/stretchr/testify/require"
	"golang.org/x/sys/unix"
)

func TestENOSPC(t *testing.T) {
	dir, err := os.MkdirTemp(t.TempDir(), "ramdisk")
	require.NoError(t, err)

	f, err := os.CreateTemp(t.TempDir(), "ramdisk_*")
	require.NoError(t, err)

	err = unix.Mount(f.Name(), dir, "tmpfs", 0, "size=1M")
	if errors.Is(err, unix.EPERM) {
		t.Skipf("skip size tests: no permission to mount: %v", err)
		return
	}
	require.NoError(t, err)
	defer func() {
		require.NoError(t, unix.Unmount(dir, 0))
	}()

	fst := New(WithPath(dir), WithDepth(1))
	require.NoError(t, fst.Open(mode.ComponentReadWrite))
	require.NoError(t, fst.Init())

	_, err = fst.Put(context.Background(), common.PutPrm{
		RawData: make([]byte, 10<<20),
	})
	require.ErrorIs(t, err, common.ErrNoSpace)
}
