package engine

import (
	"context"
	"testing"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/internal/testutil"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/shard"
	cidtest "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id/test"
	objectSDK "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object"
	oid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object/id"
	"github.com/stretchr/testify/require"
)

func TestHeadRaw(t *testing.T) {
	cnr := cidtest.ID()
	splitID := objectSDK.NewSplitID()

	parent := testutil.GenerateObjectWithCID(cnr)
	testutil.AddAttribute(parent, "foo", "bar")

	var parentAddr oid.Address
	parentAddr.SetContainer(cnr)

	idParent, _ := parent.ID()
	parentAddr.SetObject(idParent)

	child := testutil.GenerateObjectWithCID(cnr)
	child.SetParent(parent)
	child.SetParentID(idParent)
	child.SetSplitID(splitID)

	link := testutil.GenerateObjectWithCID(cnr)
	link.SetParent(parent)
	link.SetParentID(idParent)

	idChild, _ := child.ID()
	link.SetChildren(idChild)
	link.SetSplitID(splitID)

	t.Run("virtual object split in different shards", func(t *testing.T) {
		te := testNewEngine(t).setShardsNum(t, 2).prepare(t)
		e := te.engine
		defer func() { require.NoError(t, e.Close(context.Background())) }()

		s1, s2 := te.shards[0], te.shards[1]

		var putPrmLeft shard.PutPrm
		putPrmLeft.SetObject(child)

		var putPrmLink shard.PutPrm
		putPrmLink.SetObject(link)

		// put most left object in one shard
		_, err := s1.Put(context.Background(), putPrmLeft)
		require.NoError(t, err)

		// put link object in another shard
		_, err = s2.Put(context.Background(), putPrmLink)
		require.NoError(t, err)

		// head with raw flag should return SplitInfoError
		var headPrm HeadPrm
		headPrm.WithAddress(parentAddr)
		headPrm.WithRaw(true)

		_, err = e.Head(context.Background(), headPrm)
		require.Error(t, err)

		var si *objectSDK.SplitInfoError
		require.ErrorAs(t, err, &si)

		// SplitInfoError should contain info from both shards
		require.Equal(t, splitID, si.SplitInfo().SplitID())

		id1, _ := child.ID()
		id2, _ := si.SplitInfo().LastPart()
		require.Equal(t, id1, id2)

		id1, _ = link.ID()
		id2, _ = si.SplitInfo().Link()
		require.Equal(t, id1, id2)
	})
}
