package frostfsid

import (
	"fmt"

	frostfsidcore "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/core/frostfsid"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/client"
	"github.com/nspcc-dev/neo-go/pkg/encoding/fixedn"
	"github.com/nspcc-dev/neo-go/pkg/util"
)

// Client is a wrapper over StaticClient
// which makes calls with the names and arguments
// of the FrostFS ID contract.
//
// Working client must be created via constructor New.
// Using the Client that has been created with new(Client)
// expression (or just declaring a Client variable) is unsafe
// and can lead to panic.
type Client struct {
	client *client.StaticClient // static FrostFS ID contract client
}

var _ frostfsidcore.SubjectProvider = (*Client)(nil)

// NewFromMorph wraps client to work with FrostFS ID contract.
func NewFromMorph(cli *client.Client, contract util.Uint160, fee fixedn.Fixed8) (*Client, error) {
	sc, err := client.NewStatic(cli, contract, fee, client.TryNotary(), client.AsAlphabet())
	if err != nil {
		return nil, fmt.Errorf("create 'frostfsid' contract client: %w", err)
	}

	return &Client{client: sc}, nil
}
