package balance

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-contract/rpcclient/balance"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	"github.com/nspcc-dev/neo-go/pkg/core/state"
	"github.com/nspcc-dev/neo-go/pkg/util"
)

// Lock structure of balance.Lock notification from morph chain.
type Lock struct {
	id     []byte
	user   util.Uint160
	lock   util.Uint160
	amount int64 // Fixed16
	until  int64

	// txHash is used in notary environmental
	// for calculating unique but same for
	// all notification receivers values.
	txHash util.Uint256
}

// MorphEvent implements Neo:Morph Event interface.
func (Lock) MorphEvent() {}

// ID is a withdraw transaction hash.
func (l Lock) ID() []byte { return l.id }

// User returns withdraw receiver script hash from main net.
func (l Lock) User() util.Uint160 { return l.user }

// LockAccount return script hash for balance contract wallet.
func (l Lock) LockAccount() util.Uint160 { return l.lock }

// Amount of the locked assets.
func (l Lock) Amount() int64 { return l.amount }

// Until is an epoch before locked account exists.
func (l Lock) Until() int64 { return l.until }

// TxHash returns hash of the TX with lock
// notification.
func (l Lock) TxHash() util.Uint256 { return l.txHash }

// ParseLock from notification into lock structure.
func ParseLock(e *state.ContainedNotificationEvent) (event.Event, error) {
	var le balance.LockEvent
	if err := le.FromStackItem(e.Item); err != nil {
		return nil, fmt.Errorf("parse balance.LockEvent: %w", err)
	}

	return Lock{
		id:     le.TxID,
		user:   le.From,
		lock:   le.To,
		amount: le.Amount.Int64(),
		until:  le.Until.Int64(),
		txHash: e.Container,
	}, nil
}
