package container

import (
	"crypto/sha256"
	"testing"

	cidtest "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id/test"
	"github.com/nspcc-dev/neo-go/pkg/vm/stackitem"
	"github.com/stretchr/testify/require"
)

func TestParseDeleteSuccess(t *testing.T) {
	t.Run("wrong number of parameters", func(t *testing.T) {
		prms := []stackitem.Item{
			stackitem.NewMap(),
			stackitem.NewMap(),
		}

		_, err := ParseDeleteSuccess(createNotifyEventFromItems(prms))
		require.Error(t, err)
	})

	t.Run("wrong container parameter", func(t *testing.T) {
		_, err := ParseDeleteSuccess(createNotifyEventFromItems([]stackitem.Item{
			stackitem.NewMap(),
		}))

		require.Error(t, err)

		_, err = ParseDeleteSuccess(createNotifyEventFromItems([]stackitem.Item{
			stackitem.NewByteArray([]byte{1, 2, 3}),
		}))

		require.Error(t, err)
	})

	t.Run("correct behavior", func(t *testing.T) {
		id := cidtest.ID()

		binID := make([]byte, sha256.Size)
		id.Encode(binID)

		ev, err := ParseDeleteSuccess(createNotifyEventFromItems([]stackitem.Item{
			stackitem.NewByteArray(binID),
		}))

		require.NoError(t, err)

		require.Equal(t, DeleteSuccess{
			ID: id,
		}, ev)
	})
}
