package frostfs

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-contract/rpcclient/frostfs"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	"github.com/nspcc-dev/neo-go/pkg/core/state"
	"github.com/nspcc-dev/neo-go/pkg/util"
)

// Cheque structure of frostfs.Cheque notification from mainnet chain.
type Cheque struct {
	IDValue     []byte
	AmountValue int64 // Fixed8
	UserValue   util.Uint160
	LockValue   util.Uint160
}

// MorphEvent implements Neo:Morph Event interface.
func (Cheque) MorphEvent() {}

// ID is a withdraw transaction hash.
func (c Cheque) ID() []byte { return c.IDValue }

// User returns withdraw receiver script hash from main net.
func (c Cheque) User() util.Uint160 { return c.UserValue }

// Amount of the sent assets.
func (c Cheque) Amount() int64 { return c.AmountValue }

// LockAccount return script hash for balance contract wallet.
func (c Cheque) LockAccount() util.Uint160 { return c.LockValue }

// ParseCheque from notification into cheque structure.
func ParseCheque(e *state.ContainedNotificationEvent) (event.Event, error) {
	var ce frostfs.ChequeEvent
	if err := ce.FromStackItem(e.Item); err != nil {
		return nil, fmt.Errorf("parse frostfs.ChequeEvent: %w", err)
	}

	lock, err := util.Uint160DecodeBytesBE(ce.LockAccount)
	if err != nil {
		return nil, fmt.Errorf("parse frostfs.ChequeEvent: field LockAccount: %w", err)
	}

	return Cheque{
		IDValue:     ce.Id,
		AmountValue: ce.Amount.Int64(),
		UserValue:   ce.User,
		LockValue:   lock,
	}, nil
}
