package frostfs

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-contract/rpcclient/frostfs"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	"github.com/nspcc-dev/neo-go/pkg/core/state"
	"github.com/nspcc-dev/neo-go/pkg/util"
)

type Config struct {
	KeyValue   []byte
	ValueValue []byte
	IDValue    []byte

	// TxHashValue is used in notary environmental
	// for calculating unique but same for
	// all notification receivers values.
	TxHashValue util.Uint256
}

// TxHash returns hash of the TX with new epoch
// notification.
func (u Config) TxHash() util.Uint256 {
	return u.TxHashValue
}

// MorphEvent implements Neo:Morph Event interface.
func (Config) MorphEvent() {}

func (u Config) ID() []byte { return u.IDValue }

func (u Config) Key() []byte { return u.KeyValue }

func (u Config) Value() []byte { return u.ValueValue }

func ParseConfig(e *state.ContainedNotificationEvent) (event.Event, error) {
	var sce frostfs.SetConfigEvent
	if err := sce.FromStackItem(e.Item); err != nil {
		return nil, fmt.Errorf("parse frostfs.SetConfigEvent: %w", err)
	}

	return Config{
		KeyValue:    sce.Key,
		ValueValue:  sce.Value,
		IDValue:     sce.Id,
		TxHashValue: e.Container,
	}, nil
}
