package frostfs

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-contract/rpcclient/frostfs"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	"github.com/nspcc-dev/neo-go/pkg/core/state"
	"github.com/nspcc-dev/neo-go/pkg/util"
)

// Deposit structure of frostfs.Deposit notification from mainnet chain.
type Deposit struct {
	IDValue     []byte
	AmountValue int64 // Fixed8
	FromValue   util.Uint160
	ToValue     util.Uint160
}

// MorphEvent implements Neo:Morph Event interface.
func (Deposit) MorphEvent() {}

// ID is a deposit transaction hash.
func (d Deposit) ID() []byte { return d.IDValue }

// From is a script hash of asset sender in main net.
func (d Deposit) From() util.Uint160 { return d.FromValue }

// To is a script hash of asset receiver in balance contract.
func (d Deposit) To() util.Uint160 { return d.ToValue }

// Amount of transferred assets.
func (d Deposit) Amount() int64 { return d.AmountValue }

// ParseDeposit notification into deposit structure.
func ParseDeposit(e *state.ContainedNotificationEvent) (event.Event, error) {
	var de frostfs.DepositEvent
	if err := de.FromStackItem(e.Item); err != nil {
		return nil, fmt.Errorf("parse frostfs.DepositEvent: %w", err)
	}

	return Deposit{
		IDValue:     de.TxHash[:],
		AmountValue: de.Amount.Int64(),
		FromValue:   de.From,
		ToValue:     de.Receiver,
	}, nil
}
