package netmap

import (
	"git.frostfs.info/TrueCloudLab/frostfs-contract/rpcclient/netmap"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	"github.com/nspcc-dev/neo-go/pkg/core/state"
	"github.com/nspcc-dev/neo-go/pkg/util"
)

// NewEpoch is a new epoch Neo:Morph event.
type NewEpoch struct {
	Num uint64

	// Hash is used in notary environmental
	// for calculating unique but same for
	// all notification receivers values.
	Hash util.Uint256
}

// MorphEvent implements Neo:Morph Event interface.
func (NewEpoch) MorphEvent() {}

// EpochNumber returns new epoch number.
func (s NewEpoch) EpochNumber() uint64 {
	return s.Num
}

// TxHash returns hash of the TX with new epoch
// notification.
func (s NewEpoch) TxHash() util.Uint256 {
	return s.Hash
}

// ParseNewEpoch is a parser of new epoch notification event.
//
// Result is type of NewEpoch.
func ParseNewEpoch(e *state.ContainedNotificationEvent) (event.Event, error) {
	var nee netmap.NewEpochEvent
	if err := nee.FromStackItem(e.Item); err != nil {
		return nil, err
	}

	return NewEpoch{
		Num:  nee.Epoch.Uint64(),
		Hash: e.Container,
	}, nil
}
