package netmap

import (
	"crypto/elliptic"
	"errors"
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event"
	"github.com/nspcc-dev/neo-go/pkg/crypto/keys"
	"github.com/nspcc-dev/neo-go/pkg/vm/opcode"
)

var errNilPubKey = errors.New("public key is nil")

func (s *UpdatePeer) setPublicKey(v []byte) (err error) {
	if v == nil {
		return errNilPubKey
	}

	s.PubKey, err = keys.NewPublicKeyFromBytes(v, elliptic.P256())
	if err != nil {
		return fmt.Errorf("parse public key: %w", err)
	}

	return
}

const (
	// UpdateStateNotaryEvent is method name for netmap state updating
	// operations in `Netmap` contract. Is used as identificator for
	// notary delete container requests.
	UpdateStateNotaryEvent = "updateState"
)

// ParseUpdatePeerNotary from NotaryEvent into netmap event structure.
func ParseUpdatePeerNotary(ne event.NotaryEvent) (event.Event, error) {
	var (
		ev  UpdatePeer
		err error

		currCode opcode.Opcode
	)

	fieldNum := 0

	for _, op := range ne.Params() {
		currCode = op.Code()

		switch {
		case fieldNum == 0 && opcode.PUSHDATA1 <= currCode && currCode <= opcode.PUSHDATA4:
			err = ev.setPublicKey(op.Param())
			if err != nil {
				return nil, err
			}

			fieldNum++
		case fieldNum == 1:
			state, err := event.IntFromOpcode(op)
			if err != nil {
				return nil, err
			}

			err = ev.decodeState(state)
			if err != nil {
				return nil, err
			}

			fieldNum++
		case fieldNum == expectedItemNumUpdatePeer:
			return nil, event.UnexpectedArgNumErr(UpdateStateNotaryEvent)
		default:
			return nil, event.UnexpectedOpcode(UpdateStateNotaryEvent, currCode)
		}
	}

	ev.Request = ne.Raw()

	return ev, nil
}
