package control

import (
	"context"
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/engine"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control/server/ctrlmessage"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

func (s *Server) StartShardRebuild(ctx context.Context, req *control.StartShardRebuildRequest) (*control.StartShardRebuildResponse, error) {
	err := s.isValidRequest(req)
	if err != nil {
		return nil, status.Error(codes.PermissionDenied, err.Error())
	}

	if req.GetBody().GetConcurrencyLimit() == 0 || req.GetBody().GetConcurrencyLimit() > 10000 {
		return nil, status.Error(codes.InvalidArgument, fmt.Sprintf("concurrency limit must be in range (0; 10 000], current value %d", req.GetBody().GetConcurrencyLimit()))
	}

	if req.GetBody().GetTargetFillPercent() == 0 || req.GetBody().GetTargetFillPercent() > 100 {
		return nil, status.Error(codes.InvalidArgument, fmt.Sprintf("fill percent must be in range (0; 100], current value %d", req.GetBody().GetTargetFillPercent()))
	}

	prm := engine.RebuildPrm{
		ShardIDs:          s.getShardIDList(req.GetBody().GetShard_ID()),
		ConcurrencyLimit:  req.GetBody().GetConcurrencyLimit(),
		TargetFillPercent: req.GetBody().GetTargetFillPercent(),
	}

	res, err := s.s.Rebuild(ctx, prm)
	if err != nil {
		return nil, status.Error(codes.Internal, err.Error())
	}

	resp := &control.StartShardRebuildResponse{Body: &control.StartShardRebuildResponse_Body{}}
	for _, r := range res.ShardResults {
		if r.Success {
			resp.Body.Results = append(resp.GetBody().GetResults(), control.StartShardRebuildResponse_Body_Status{
				Shard_ID: *r.ShardID,
				Success:  true,
			})
		} else {
			resp.Body.Results = append(resp.GetBody().GetResults(), control.StartShardRebuildResponse_Body_Status{
				Shard_ID: *r.ShardID,
				Error:    r.ErrorMsg,
			})
		}
	}

	err = ctrlmessage.Sign(s.key, resp)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
