package replicator

import (
	"context"
	"errors"

	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/logs"
	containerCore "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/core/container"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/engine"
	"git.frostfs.info/TrueCloudLab/frostfs-observability/tracing"
	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/trace"
	"go.uber.org/zap"
)

var errObjectNotDefined = errors.New("object is not defined")

func (p *Replicator) HandleLocalPutTask(ctx context.Context, task Task) {
	p.metrics.IncInFlightRequest()
	defer p.metrics.DecInFlightRequest()
	defer func() {
		p.log.Debug(ctx, logs.ReplicatorFinishWork, zap.String("type", "pull"))
	}()

	ctx, span := tracing.StartSpanFromContext(ctx, "Replicator.HandleLocalPutTask",
		trace.WithAttributes(
			attribute.Stringer("address", task.Addr),
			attribute.Int("nodes_count", len(task.Nodes)),
		))
	defer span.End()

	if task.Obj == nil {
		p.log.Error(ctx, logs.ReplicatorCouldNotPutObjectToLocalStorage,
			zap.Stringer("object", task.Addr),
			zap.Error(errObjectNotDefined))
		return
	}

	err := engine.Put(ctx, p.localStorage, task.Obj, containerCore.IsIndexedContainer(task.Container))
	if err != nil {
		p.log.Error(ctx, logs.ReplicatorCouldNotPutObjectToLocalStorage,
			zap.Stringer("object", task.Addr),
			zap.Error(err))
	}
}
